@@ -57,6 +57,18 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
+
+
+#define HEADING_FLAG(HEADING) __ ## HEADING ## _print_done
+#define HEADING_HDR(HEADING) HEADING ## _header
+#define HEADING_PRINT_COND(HEADING)	        \
+  do {                                          \
+      if (!HEADING_FLAG(HEADING)) {		\
+	  printf("%s", HEADING_HDR(HEADING));	\
+	  HEADING_FLAG(HEADING) = 1;		\
+      }					        \
+  } while(0)
+
 /* -------------------------------------------------------------------
  * usage
  * ------------------------------------------------------------------- */
@@ -80,12 +92,18 @@ extern const char client_port[];
 
 extern const char client_report_epoch_start[];
 
+extern const char client_report_epoch_start_current[];
+
 extern const char server_pid_port[];
 
 extern const char client_pid_port[];
 
+extern const char client_pid_port_dev[];
+
 extern const char bind_address[];
 
+extern const char bind_address_iface[];
+
 extern const char multicast_ttl[];
 
 extern const char join_multicast[];
@@ -112,7 +130,7 @@ extern const char window_default[];
 
 extern const char wait_server_threads[];
 
-extern const char client_udp_isochronous[];
+extern const char client_isochronous[];
 
 extern const char client_fq_pacing[];
 /* -------------------------------------------------------------------
@@ -121,10 +139,12 @@ extern const char client_fq_pacing[];
 
 extern const char report_bw_header[];
 
+extern const char report_bw_sumcnt_header[];
+
 extern const char report_bw_format[];
 
 extern const char report_sum_bw_format[];
-
+extern const char report_sumcnt_bw_format[];
 extern const char report_bw_read_format[];
 
 extern const char report_bw_jitter_loss_header[];
@@ -133,9 +153,12 @@ extern const char report_bw_jitter_loss_format[];
 
 extern const char report_sum_bw_jitter_loss_format[];
 
+extern const char report_sumcnt_bw_jitter_loss_format[];
+
 /* -------------------------------------------------------------------
  * Enhanced reports (per -e)
  * ------------------------------------------------------------------- */
+
 extern const char client_write_size[];
 
 extern const char server_read_size[];
@@ -146,18 +169,30 @@ extern const char report_sum_bw_enhanced_format[];
 
 extern const char report_bw_read_enhanced_header[];
 
+extern const char report_bw_read_enhanced_netpwr_header[];
+
+extern const char report_bw_read_enhanced_netpwr_format[];
+
 extern const char report_bw_read_enhanced_format[];
 
 extern const char report_sum_bw_read_enhanced_format[];
 
+extern const char report_sumcnt_bw_read_enhanced_format[];
+
 extern const char report_triptime_enhanced_format[];
 
 extern const char report_bw_write_enhanced_header[];
 
+extern const char report_bw_write_sumcnt_enhanced_header[];
+
 extern const char report_bw_write_enhanced_format[];
 
+extern const char report_bw_write_enhanced_nocwnd_format[];
+
 extern const char report_sum_bw_write_enhanced_format[];
 
+extern const char report_sumcnt_bw_write_enhanced_format[];
+
 extern const char report_bw_pps_enhanced_header[];
 
 extern const char report_bw_pps_enhanced_format[];
@@ -168,6 +203,10 @@ extern const char report_bw_pps_enhanced_isoch_format[];
 
 extern const char report_sum_bw_pps_enhanced_format[];
 
+extern const char report_bw_jitter_loss_pps_header[];
+
+extern const char report_bw_jitter_loss_pps_format[];
+
 extern const char report_bw_jitter_loss_enhanced_header[];
 
 extern const char report_bw_jitter_loss_enhanced_format[];
@@ -180,6 +219,29 @@ extern const char report_bw_jitter_loss_suppress_enhanced_format[];
 
 extern const char report_sum_bw_jitter_loss_enhanced_format[];
 
+extern const char report_sumcnt_bw_jitter_loss_enhanced_format[];
+
+extern const char report_bw_sum_fullduplex_format[];
+
+extern const char report_bw_sum_fullduplex_enhanced_format[];
+
+extern const char report_frame_jitter_loss_enhanced_header[];
+
+extern const char report_frame_jitter_loss_enhanced_format[];
+
+extern const char report_frame_jitter_loss_suppress_enhanced_format[];
+
+extern const char report_frame_tcp_enhanced_header[];
+
+extern const char report_udp_fullduplex_header[];
+
+extern const char report_udp_fullduplex_format[];
+
+extern const char report_udp_fullduplex_sum_format[];
+
+extern const char report_udp_fullduplex_enhanced_format[];
+
+
 /* -------------------------------------------------------------------
  * Misc reports
  * ------------------------------------------------------------------- */
@@ -192,12 +254,16 @@ extern const char report_sum_outoforder[];
 
 extern const char report_peer[];
 
+extern const char report_peer_dev[];
+
 extern const char report_mss_unsupported[];
 
 extern const char report_mss[];
 
 extern const char report_datagrams[];
 
+extern const char report_sumcnt_datagrams[];
+
 extern const char report_sum_datagrams[];
 
 extern const char server_reporting[];
