@@ -72,6 +72,14 @@ public:
         setnow();
     }
 
+    /* -------------------------------------------------------------------
+     * Copy construcutor
+     * ------------------------------------------------------------------- */
+    Timestamp(const Timestamp &t2) {
+        mTime.tv_sec = t2.mTime.tv_sec;
+        mTime.tv_usec = t2.mTime.tv_usec;
+    }
+
     /* -------------------------------------------------------------------
      * Create a timestamp, with the given seconds/microseconds
      * ------------------------------------------------------------------- */
@@ -89,14 +97,14 @@ public:
     /* -------------------------------------------------------------------
      * Set timestamp to current time.
      * ------------------------------------------------------------------- */
-    void setnow( void ) {
+    void inline setnow( void ) {
 #ifdef HAVE_CLOCK_GETTIME
 	struct timespec t1;
 	clock_gettime(CLOCK_REALTIME, &t1);
 	mTime.tv_sec  = t1.tv_sec;
         mTime.tv_usec = t1.tv_nsec / 1000;
 #else
-	gettimeofday( &mTime, NULL );
+	gettimeofday(&mTime, NULL);
 #endif
     }
 
@@ -122,14 +130,14 @@ public:
     /* -------------------------------------------------------------------
      * return seconds portion of timestamp
      * ------------------------------------------------------------------- */
-    long getSecs( void ) {
+    long inline getSecs( void ) {
         return mTime.tv_sec;
     }
 
     /* -------------------------------------------------------------------
      * return microseconds portion of timestamp
      * ------------------------------------------------------------------- */
-    long getUsecs( void ) {
+    long inline getUsecs( void ) {
         return mTime.tv_usec;
     }
 
@@ -225,6 +233,16 @@ public:
                 mTime.tv_usec <  kMillion );
     }
 
+    /* -------------------------------------------------------------------
+     * add micro seconds to my timestamp.
+     * ------------------------------------------------------------------- */
+    void add(unsigned int usec ) {
+        mTime.tv_usec += usec;
+	mTime.tv_sec += mTime.tv_usec / kMillion;
+	mTime.tv_usec = mTime.tv_usec % kMillion;
+	// assert((mTime.tv_usec >= 0) && (mTime.tv_usec < kMillion));
+    }
+
     /* -------------------------------------------------------------------
      * return true if my timestamp is before the right timestamp.
      * ------------------------------------------------------------------- */
