@@ -71,9 +71,6 @@
     #endif
 #endif /* HAVE_CONFIG_H */
 
-/* turn off assert debugging */
-#define NDEBUG
-
 /* standard C headers */
 #include <stdlib.h>
 #include <stdio.h>
@@ -86,6 +83,41 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <inttypes.h>
+#include <limits.h>
+
+// AF_PACKET HEADERS
+#if defined(HAVE_LINUX_FILTER_H) && defined(HAVE_AF_PACKET)
+#include <net/ethernet.h>
+// Bummer, AF_PACKET requires kernel headers as <netpacket/packet.h> isn't sufficient
+#include <linux/filter.h>
+#include <linux/if_packet.h>
+#include <linux/ip.h>
+#include <linux/udp.h>
+// In older linux kernels, the kernel headers and glibc headers are in conflict,
+// specificially <linux/in6.h> and <netinit/in.h>
+// preventing the following include:
+//   #include <linux/ipv6.h> (which includes <linux/in6.h>)
+//   and the use of sizeof(struct ipv6hdr)
+//
+// See https://patchwork.ozlabs.org/patch/631400/
+//
+// "In upstream kernel 56c176c9 the _UAPI prefix was stripped and
+// this broke our synchronized headers again. We now need to check
+// for _LINUX_IN6_H and _IPV6_H, and keep checking the old versions
+// of the header guard checks for maximum backwards compatibility
+// with older Linux headers (the history is actually a bit muddled
+// here and it appears upstream linus kernel broke this 10 months
+// *before* our fix was ever applied to glibc, but without glibc
+// testing we didn't notice and distro kernels have their own
+// testing to fix this)."
+//
+// The use cases fixed by this patch are:
+// #include <linux/in6.h>
+// #include <netinet/in.h>
+//
+// force the ipv6hdr length to 40 vs use of sizeof(struct ipv6hdr)
+#define  IPV6HDRLEN 40
+#endif // HAVE_AF_PACKET
 
 #ifdef WIN32
 
@@ -198,27 +230,16 @@ typedef struct sockaddr_in iperf_sockaddr;
     #endif
 //#endif
 
-// Rationalize stdint definitions and sizeof, thanks to ac_create_stdint_h.m4
-// from the gnu archive
+// inttypes.h is already included
 
-#include <iperf-int.h>
-#ifdef HAVE_QUAD_SUPPORT
-#ifdef HAVE_INT64_T
-typedef int64_t max_size_t;
-typedef u_int64_t umax_size_t;
+#ifdef HAVE_FASTSAMPLING
+#define IPERFTimeFrmt "%4.4f-%4.4f"
+#define IPERFTimeSpace "            "
 #else
-typedef long long max_size_t;
-typedef unsigned long long umax_size_t;
-#endif // 64
-#else
-#ifdef HAVE_INT32_T
-typedef int32_t max_size_t;
-typedef u_int32_t umax_size_t;
-#else
-typedef long max_size_t;
-typedef unsigned long umax_size_t;
-#endif // 32
+#define IPERFTimeFrmt "%4.2f-%4.2f"
+#define IPERFTimeSpace "        "
 #endif
+
 /* in case the OS doesn't have these, we provide our own implementations */
 #include "gettimeofday.h"
 #include "inet_aton.h"
@@ -230,10 +251,9 @@ typedef unsigned long umax_size_t;
     #define SHUT_RDWR 2
 #endif // SHUT_RD
 
+
+/* Internal debug */
+//#define INITIAL_PACKETID 0x7FFFFF00LL
+//#define SHOW_PACKETID
+
 #endif /* HEADERS_H */
-
-
-
-
-
-
