@@ -61,14 +61,13 @@
 #define HEADERS()
 
 #include "headers.h"
-
 #include "Settings.hpp"
 #include "PerfSocket.hpp"
 #include "Locale.h"
 #include "Condition.h"
 #include "Timestamp.hpp"
 #include "Listener.hpp"
-#include "List.h"
+#include "active_hosts.h"
 #include "util.h"
 
 #ifdef WIN32
@@ -79,7 +78,7 @@
  * prototypes
  * ------------------------------------------------------------------- */
 // Function called at exit to clean up as much as possible
-void cleanup( void );
+void cleanup(void);
 
 /* -------------------------------------------------------------------
  * global variables
@@ -92,22 +91,27 @@ extern "C" {
     int groupID = 0;
     // Mutex to protect access to the above ID
     Mutex groupCond;
-    // Condition used to signify advances of the current
-    // records being accessed in a report and also to
-    // serialize modification of the report list
-    Condition ReportCond;
-    Condition ReportDoneCond;
+    // Condition used to signal the reporter thread
+    // when a packet ring is full.  Shouldn't really
+    // be needed but is "belts and suspeners"
+    struct Condition ReportCond;
+    // Initialize reporter thread mutex
+    struct AwaitMutex reporter_state;
+    struct AwaitMutex threads_start;
+    struct BarrierMutex transmits_start;
 }
 
+
 // global variables only accessed within this file
 
 // Thread that received the SIGTERM or SIGINT signal
 // Used to ensure that if multiple threads receive the
 // signal we do not prematurely exit
 nthread_t sThread;
+static thread_Settings* ext_gSettings;
 // The main thread uses this function to wait
 // for all other threads to complete
-void waitUntilQuit( void );
+void waitUntilQuit(void);
 
 /* -------------------------------------------------------------------
  * main()
@@ -119,52 +123,65 @@ void waitUntilQuit( void );
  * starts up server or client thread
  * waits for all threads to complete
  * ------------------------------------------------------------------- */
-int main( int argc, char **argv ) {
+int main(int argc, char **argv) {
 
     // Set SIGTERM and SIGINT to call our user interrupt function
-    my_signal( SIGTERM, Sig_Interupt );
-    my_signal( SIGINT,  Sig_Interupt );
+    my_signal(SIGTERM, Sig_Interupt);
+    my_signal(SIGINT,  Sig_Interupt);
 #ifndef WIN32
-    my_signal( SIGALRM,  Sig_Interupt );
+    my_signal(SIGALRM,  Sig_Interupt);
 
     // Ignore broken pipes
     signal(SIGPIPE,SIG_IGN);
 #else
     // Start winsock
     WSADATA wsaData;
-    int rc = WSAStartup( 0x202, &wsaData );
-    WARN_errno( rc == SOCKET_ERROR, "WSAStartup" );
-	if (rc == SOCKET_ERROR)
-		return 0;
+    int rc = WSAStartup(0x202, &wsaData);
+    WARN_errno(rc == SOCKET_ERROR, "WSAStartup");
+    if (rc == SOCKET_ERROR)
+	return 0;
 
     // Tell windows we want to handle our own signals
-    SetConsoleCtrlHandler( sig_dispatcher, true );
+    SetConsoleCtrlHandler(sig_dispatcher, true);
 #endif
 
     // Initialize global mutexes and conditions
-    Condition_Initialize ( &ReportCond );
-    Condition_Initialize ( &ReportDoneCond );
-    Mutex_Initialize( &groupCond );
-    Mutex_Initialize( &clients_mutex );
+    Iperf_initialize_active_table();
+    Condition_Initialize (&ReportCond);
+
+#ifdef HAVE_THREAD_DEBUG
+    Mutex_Initialize(&packetringdebug_mutex);
+    Mutex_Initialize(&thread_debug_mutex);
+#endif
+    // Initialize reporter thread mutex
+    reporter_state.ready = 0;
+    threads_start.ready = 0;
+    transmits_start.count = 0;
+    Condition_Initialize(&reporter_state.await);
+    Condition_Initialize(&threads_start.await);
+    Condition_Initialize(&transmits_start.await);
 
     // Initialize the thread subsystem
-    thread_init( );
+    thread_init();
 
     // Initialize the interrupt handling thread to 0
     sThread = thread_zeroid();
 
     // perform any cleanup when quitting Iperf
-    atexit( cleanup );
+    atexit(cleanup);
 
     // Allocate the "global" settings
-    thread_Settings* ext_gSettings = new thread_Settings;
+    ext_gSettings = new thread_Settings;
 
+    // Default reporting mode here to avoid unitialized warnings
+    // this won't be the actual mode
+    ThreadMode ReporterThreadMode = kMode_Reporter;
     // Initialize settings to defaults
-    Settings_Initialize( ext_gSettings );
+    Settings_Initialize(ext_gSettings);
     // read settings from environment variables
-    Settings_ParseEnvironment( ext_gSettings );
+    Settings_ParseEnvironment(ext_gSettings);
     // read settings from command-line parameters
-    Settings_ParseCommandLine( argc, argv, ext_gSettings );
+    Settings_ParseCommandLine(argc, argv, ext_gSettings);
 
     // Check for either having specified client or server
     if ((ext_gSettings->mThreadMode != kMode_Client) && (ext_gSettings->mThreadMode != kMode_Listener)) {
@@ -183,51 +200,44 @@ int main( int argc, char **argv ) {
 
 	// starting the service by SCM, there is no arguments will be passed in.
 	// the arguments will pass into Service_Main entry.
-        if (!StartServiceCtrlDispatcher(dispatchTable) )
+        if (!StartServiceCtrlDispatcher(dispatchTable))
             // If the service failed to start then print usage
 #endif
-	    fprintf( stderr, usage_short, argv[0], argv[0] );
+	    fprintf(stderr, usage_short, argv[0], argv[0]);
 	return 0;
     }
 
-
+    unsetReport(ext_gSettings);
     switch (ext_gSettings->mThreadMode) {
     case kMode_Client :
-	if ( isDaemon( ext_gSettings ) ) {
+	if (isDaemon(ext_gSettings)) {
 	    fprintf(stderr, "Iperf client cannot be run as a daemon\n");
 	    return 0;
 	}
         // initialize client(s)
-        client_init( ext_gSettings );
-#ifdef HAVE_CLOCK_NANOSLEEP
-#ifdef HAVE_CLOCK_GETTIME
-	if (isEnhanced(ext_gSettings) && isTxStartTime(ext_gSettings)) {
-	    struct timespec t1;
-	    clock_gettime(CLOCK_REALTIME, &t1);
-	    fprintf(stdout, "Client thread(s) traffic start time %ld.%.9ld current time is %ld.%.9ld (epoch/unix format)\n",ext_gSettings->txstart.tv_sec, ext_gSettings->txstart.tv_nsec, t1.tv_sec, t1.tv_nsec);
-	}
-#endif
-#endif
+	transmits_start.count = ext_gSettings->mThreads;
+	ext_gSettings->connects_done = &transmits_start;
+        client_init(ext_gSettings);
+	ReporterThreadMode = kMode_ReporterClient;
 	break;
     case kMode_Listener :
 #ifdef WIN32
-     // Remove the Windows service if requested
-	if ( isRemoveService( ext_gSettings ) ) {
+	// Remove the Windows service if requested
+	if (isRemoveService(ext_gSettings)) {
 	    // remove the service
-	    if ( CmdRemoveService() ) {
+	    if (CmdRemoveService()) {
 		fprintf(stderr, "IPerf Service is removed.\n");
 	    }
 	}
-	if ( isDaemon( ext_gSettings ) ) {
+	if (isDaemon(ext_gSettings)) {
 	    CmdInstallService(argc, argv);
 	} else if (isRemoveService(ext_gSettings)) {
 	    return 0;
 	}
 #else // *nix system
-	if ( isDaemon( ext_gSettings ) ) {
-	    fprintf( stderr, "Running Iperf Server as a daemon\n");
+	if (isDaemon(ext_gSettings)) {
+	    fprintf(stderr, "Running Iperf Server as a daemon\n");
 	    // Start the server as a daemon
-	    fflush(stderr);
 	    // redirect stdin, stdout and sterr to /dev/null (see dameon and no close flag)
 	    if (daemon(1, 0) < 0) {
 	        perror("daemon");
@@ -236,26 +246,28 @@ int main( int argc, char **argv ) {
 #endif
 	break;
     default :
-	fprintf( stderr, "unknown mode");
+	fprintf(stderr, "unknown mode");
 	break;
     }
 #ifdef HAVE_THREAD
-        // start up the reporter and client(s) or listener
+    // Last step is to initialize the reporter then start all threads
     {
 	thread_Settings *into = NULL;
 	// Create the settings structure for the reporter thread
-	Settings_Copy( ext_gSettings, &into );
-	into->mThreadMode = kMode_Reporter;
-
+	Settings_Copy(ext_gSettings, &into, 1);
+	into->mThreadMode = ReporterThreadMode;
+	into->mSumReport = NULL;
+	into->mFullDuplexReport = NULL;
 	// Have the reporter launch the client or listener
 	into->runNow = ext_gSettings;
-
 	// Start all the threads that are ready to go
-	thread_start( into );
+	thread_start_all(into);
+	threads_start.ready = 1;
+        Condition_Signal(&threads_start.await);
     }
 #else
     // No need to make a reporter thread because we don't have threads
-    thread_start( ext_gSettings );
+    thread_start(ext_gSettings);
 #endif
 
     // wait for other (client, server) threads to complete
@@ -270,15 +282,15 @@ int main( int argc, char **argv ) {
  * respond appropriately.. [static]
  * ------------------------------------------------------------------- */
 
-void Sig_Interupt( int inSigno ) {
+void Sig_Interupt(int inSigno) {
 #ifdef HAVE_THREAD
     // We try to not allow a single interrupt handled by multiple threads
     // to completely kill the app so we save off the first thread ID
     // then that is the only thread that can supply the next interrupt
-    if ( (inSigno == SIGINT) && thread_equalid( sThread, thread_zeroid() ) ) {
+    if ((inSigno == SIGINT) && thread_equalid(sThread, thread_zeroid())) {
         sThread = thread_getid();
-    } else if ( thread_equalid( sThread, thread_getid() ) ) {
-        sig_exit( inSigno );
+    } else if (thread_equalid(sThread, thread_getid())) {
+        sig_exit(inSigno);
     }
     // global variable used by threads to see if they were interrupted
     sInterupted = inSigno;
@@ -289,11 +301,11 @@ void Sig_Interupt( int inSigno ) {
 #endif
 	// with threads, stop waiting for non-terminating threads
 	// (ie Listener Thread)
-	thread_release_nonterm( inSigno );
+	thread_release_nonterm(inSigno);
 
 #else
     // without threads, just exit quietly, same as sig_exit()
-    sig_exit( inSigno );
+    sig_exit(inSigno);
 #endif
 }
 
@@ -302,16 +314,26 @@ void Sig_Interupt( int inSigno ) {
  * either by exit() or terminating main().
  * ------------------------------------------------------------------- */
 
-void cleanup( void ) {
+void cleanup(void) {
 #ifdef WIN32
     // Shutdown Winsock
     WSACleanup();
 #endif
-    // clean up the list of clients
-    Iperf_destroy ( &clients );
+    // clean up the list of active clients
+    Iperf_destroy_active_table();
+    // done actions
+    // Destroy global mutexes and conditions
 
+    Condition_Destroy (&ReportCond);
+    Condition_Destroy(&reporter_state.await);
+    Condition_Destroy(&threads_start.await);
+    Condition_Destroy(&transmits_start.await);
+#ifdef HAVE_THREAD_DEBUG
+    Mutex_Destroy(&packetringdebug_mutex);
+    Mutex_Destroy(&thread_debug_mutex);
+#endif
     // shutdown the thread subsystem
-    thread_destroy( );
+    thread_destroy();
 } // end cleanup
 
 #ifdef WIN32
@@ -327,20 +349,20 @@ VOID ServiceStart (DWORD dwArgc, LPTSTR *lpszArgv) {
 
     // report the status to the service control manager.
     //
-    if ( !ReportStatusToSCMgr(
+    if (!ReportStatusToSCMgr(
                              SERVICE_START_PENDING, // service state
                              NO_ERROR,              // exit code
-                             3000) )                 // wait hint
+                             3000))                 // wait hint
         goto clean;
 
     ext_gSettings = new thread_Settings;
 
     // Initialize settings to defaults
-    Settings_Initialize( ext_gSettings );
+    Settings_Initialize(ext_gSettings);
     // read settings from environment variables
-    Settings_ParseEnvironment( ext_gSettings );
+    Settings_ParseEnvironment(ext_gSettings);
     // read settings from command-line parameters
-    Settings_ParseCommandLine( dwArgc, lpszArgv, ext_gSettings );
+    Settings_ParseCommandLine(dwArgc, lpszArgv, ext_gSettings);
 
     // Arguments will be lost when the service is started by SCM, but
     // we need to be at least a listener
@@ -348,49 +370,49 @@ VOID ServiceStart (DWORD dwArgc, LPTSTR *lpszArgv) {
 
     // report the status to the service control manager.
     //
-    if ( !ReportStatusToSCMgr(
+    if (!ReportStatusToSCMgr(
                              SERVICE_START_PENDING, // service state
                              NO_ERROR,              // exit code
-                             3000) )                 // wait hint
+                             3000))                 // wait hint
         goto clean;
 
     // if needed, redirect the output into a specified file
-    if ( !isSTDOUT( ext_gSettings ) ) {
-        redirect( ext_gSettings->mOutputFileName );
+    if (!isSTDOUT(ext_gSettings)) {
+        redirect(ext_gSettings->mOutputFileName);
     }
 
     // report the status to the service control manager.
     //
-    if ( !ReportStatusToSCMgr(
+    if (!ReportStatusToSCMgr(
                              SERVICE_START_PENDING, // service state
                              NO_ERROR,              // exit code
-                             3000) )                 // wait hint
+                             3000))                 // wait hint
         goto clean;
 
     // initialize client(s)
-    if ( ext_gSettings->mThreadMode == kMode_Client ) {
-        client_init( ext_gSettings );
+    if (ext_gSettings->mThreadMode == kMode_Client) {
+        client_init(ext_gSettings);
     }
 
     // start up the reporter and client(s) or listener
     {
         thread_Settings *into = NULL;
 #ifdef HAVE_THREAD
-        Settings_Copy( ext_gSettings, &into );
+        Settings_Copy(ext_gSettings, &into, 1);
         into->mThreadMode = kMode_Reporter;
         into->runNow = ext_gSettings;
 #else
         into = ext_gSettings;
 #endif
-        thread_start( into );
+        thread_start_all(into);
     }
 
     // report the status to the service control manager.
     //
-    if ( !ReportStatusToSCMgr(
+    if (!ReportStatusToSCMgr(
                              SERVICE_RUNNING,       // service state
                              NO_ERROR,              // exit code
-                             0) )                    // wait hint
+                             0))                    // wait hint
         goto clean;
 
     clean:
@@ -420,7 +442,7 @@ VOID ServiceStart (DWORD dwArgc, LPTSTR *lpszArgv) {
 //
 VOID ServiceStop() {
 #ifdef HAVE_THREAD
-    Sig_Interupt( 1 );
+    Sig_Interupt(1);
 #else
     sig_exit(1);
 #endif
